GemList = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("GemList")
local chatprefix = "!gems"
local chatprefix2 = "<GemList>"
local LOCKED = 0

function GemList:OnInitialize()
	self:Reset()
	GemList:RegisterDB("GemListDB")
	GemList:RegisterDefaults("account", {
		Gems = {},
		Gemsname= {},
		Gemsdesc={},
		showincmessages=false,
		showexmessages=false,
		searchdesc=true,
		autoscan=true,
		TIMELOCK=3
		}
	)  
	
local options = { 
    type="group",
    args = {
    	inc= {
    		type="toggle",
    		name=L["Show incoming requests"],
    		desc=L["Show incoming requests"],
    		get=function() return self.db.account.showincmessages end,
    		set=function(v) self.db.account.showincmessages = v end,
    		},
			out= {
				type="toggle",
				name=L["Show your replies"],
				desc=L["Show your replies"],
				get=function() return self.db.account.showexmessages end,
				set=function(v) self.db.account.showexmessages = v end,
			},
			autoscan= {
				type="toggle",
				name=L["Autoscan"],
				desc=L["Automatically scan your Jewelcrafting panel when it opens"],
				get=function() return self.db.account.autoscan end,
				set=function(v) GemList:ToggleAutoScan(v) end,
			},
			desc= {
				type="toggle",
				name=L["Search description"],
				desc=L["Also search Gem description for a match"],
				get=function() return self.db.account.searchdesc end,
				set=function(v) self.db.account.searchdesc = v end,
			},
			scan= {
				type="execute",
				name=L["Manual scan (open your Jewelcrafting panel first)"],
				desc=L["Manual scan (open your Jewelcrafting panel first)"],
				func=function() GemList:Scan() end,
				order=101,
			},
			tlock= {
				type="range",
				name=L["Lock time between 2 requests (antispam)"],
				desc=L["Lock time between 2 requests (antispam)"],
				get=function() return self.db.account.TIMELOCK end,
    		set=function(v) self.db.account.TIMELOCK = v end,
				min=1,
				max=30,
				step=1,
			}
		}
}

GemList:RegisterChatCommand({"/glist", "/Gemlist"}, options)
end

function GemList:OnEnable()
	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
	if self.db.account.autoscan then 
		self:RegisterEvent("TRADE_SKILL_SHOW") 
	end
end

function GemList:OnDisable()
	self:Reset()
	self:UnhookAll()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:UnregisterEvent("TRADE_SKILL_SHOW")
end

function GemList:Reset()
	LOCKED = 0
end

function GemList:WhisperHandler(event) 
	if arg1 and arg1 ~= "" then
		
		if (string.sub(arg1, 1, string.len(chatprefix)) == chatprefix and 
		not self.db.account.showincmessages and 
		event == "CHAT_MSG_WHISPER") then
			return
		end
		
		if (string.sub(arg1, 1, string.len(chatprefix2)) == chatprefix2 and 
		not self.db.account.showexmessages and 
		event == "CHAT_MSG_WHISPER_INFORM") then
			return
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function GemList:CHAT_MSG_WHISPER()
	local argum = self:GetArgs(arg1, " ")
	local sender = arg2
	
	if argum[1] == chatprefix then 
		if arg6 == L["AFK"] or arg6 == L["DND"] then 
			self:Respond(L["First clear your tag: "]..arg6, sender) 
			return 
		end
		
		table.remove(argum, 1)
		-- argum = self:RecomposeLink(argum)
		
		if table.getn(argum) == 0 then 
			self:Respond(L["Usage:"], sender) 
			self:Respond(L["!gems [color]"], sender)
			 
			if self.db.account.searchdesc then 
				self:Respond(L["Searching gem description is currently enabled."], sender)
			else 
				self:Respond(L["Searching gem description is currently disabled."], sender)
			end
			
			return 
		end
		
		self:Handle(argum, sender)
	end
end

function GemList:GetArgs(text, separator)
-- copy arguments to table
	local args = {}
	local i = 0
	
	for value in string.gmatch(text, "[^"..separator.."]+") do
		i = i + 1
		args[i] = value
	end	
	
	return args
end

function GemList:Handle(argum, sender)
	if table.getn(self.db.account.Gems) == 0 then 
		self:Respond(L["Database is currently empty."], sender) 
		self:Print(sender..L[" wants to use GemList but the Jewelcrafting panel has not yet been opened."])
		return 
	end
	
	local searchresult={}
	
	for i = 1, table.getn(self.db.account.Gems) do
		local paf = 0
		local GemName = string.lower(self.db.account.GemsName[i])
		local GemDescription = string.lower(self.db.account.GemsDescription[i])
		local GemColor = string.lower(self.db.account.GemsColor[i])
		
		for j = 1, table.getn(argum) do
			if strfind(GemName, string.lower(argum[j])) then 
				paf = paf + 1 
			elseif self.db.account.searchdesc and strfind(GemDescription, string.lower(argum[j])) then 
				paf = paf + 1
			elseif strfind(GemColor, string.lower(argum[j])) then
				paf = paf + 1				
			end
		end
		 
		if paf == table.getn(argum) then 
			table.insert(searchresult, self.db.account.Gems[i]) 
		end	
	end
	
	if table.getn(searchresult) == 0 then 
		self:Respond(L["Nothing found."], sender)
	else 
		if LOCKED==0 then
			LOCKED=1
			
			for i = 1, table.getn(searchresult) do 
				self:Respond(searchresult[i], sender) 
			end 
			
			self:ScheduleEvent(function() LOCKED=0 return end, self.db.account.TIMELOCK)
		else 
			self:Respond(L["Wait ... ("]..self.db.account.TIMELOCK..L[" seconds lock (antispam))"], sender)
		end
	end
end

function GemList:Respond(message,sender)
	SendChatMessage(chatprefix2.." "..message, "WHISPER", nil, sender)
end

function GemList:TRADE_SKILL_SHOW()
	if self.db.account.autoscan then 
		if not TradeSkillFrameTitleText then 
			return 
		end
		if string.find(TradeSkillFrameTitleText:GetText(), L["Jewelcrafting"]) then 
			return 
		end
		self:Scan()
	end
end

function GemList:ToggleAutoScan(v)
	self.db.account.autoscan = v
	self:RegisterEvent("CRAFT_SHOW")
	if not v then self:UnregisterEvent("CRAFT_SHOW") end
end

function GemList:Scan()

	if not TradeSkillFrameTitleText then 
		self:Print(L["Open your Jewelcrafting panel first"]) 
		return 
	end
	
	if string.find(TradeSkillFrameTitleText:GetText(), L["Jewelcrafting"]) then 
		self:Print(L["Open your Jewelcrafting panel first"]) 
		return 
	end
	
	self.db.account.Gems = {}
	self.db.account.GemsName = {}
	self.db.account.GemsDescription = {}
	self.db.account.GemsColor = {}
	
	for i = self:GetFirstGemIndex(), GetNumTradeSkills() do
		local skillName, skillType = GetTradeSkillInfo(i)
		
		if skillType~="header" and skillName then
				
			local link = GetTradeSkillItemLink(i)
			local description = GetTradeSkillItemStats(i)
			local icon = GetTradeSkillIcon(i)
			--self:Print(link..': '..description)
			--self:Print('Icon: '..icon)
			--self:Print('Color: '..L[icon])
				
			table.insert(self.db.account.Gems, link)
			table.insert(self.db.account.GemsName, skillName)
			table.insert(self.db.account.GemsColor, L[icon])
				
			if description then 
			 	table.insert(self.db.account.GemsDescription, description) 
			else 
			  table.insert(self.db.account.GemsDescription, "") 
			end
		else
			--self:Print("End of Gems")
			return
		end
	end
	return
end


function GemList:GetFirstGemIndex()
-- finds the Gems category and expands it if needed
	for i = 1, GetNumTradeSkills() do
		local skillName, skillType, isExpanded;
		skillName, skillType, _, isExpanded = GetTradeSkillInfo(i);
		if skillType == "header" and skillName == L["Gems"] then
			if not isExpanded then 
				ExpandTradeSkillSubClass(i);
			end
			return i+1;
		end
	end
end

